
-- [Ice Project 2012, turtlesort.com]

-- This is a dedicated event manager for scene-main.
-- There are two types of event triggers to check for:
-- map triggers (for events triggered when a new map is
-- loaded) and tile triggers (for events triggered when a
-- player moves to a specific tile on a specific map).

local manager = {}

function manager.maptrigger()

	local scene = game.getScene('main')

	-- The intro
	if game.getCurrentScene() == scene and scene.currentmapindex == 1 then

		-- Events on game start up
		scene.player.direction = 'down'
	
		scene.dialog.endevent = function()
			scene.dialog.visible = false
			scene.player.direction = 'up'
			
			game.timer.execute(function()
				scene.player.tiley = scene.player.tiley - 1
			end,0.25)

			game.timer.execute(function()
				scene.player.tiley = scene.player.tiley - 2
			end,1.0)
			
			game.timer.execute(function()
				scene.dialog.endevent = function()
					scene.player.tiley = scene.player.tiley - 1
				end
				scene.dialog.display(game.text.beforefalling,3)
			end,2.0)

			-- scene.player.tiley = scene.currentmap.properties.endy
		end
		
		scene.dialog.display(game.text.opening,2)

	-- Third last map of the game, where we can encounter the Child of Winter, before fade in
	elseif game.getCurrentScene() ~= scene and scene.currentmapindex == (#scene.maps - 3) then
	
		-- Want the child of winter to appear no matter what? Set chance value to 1
		scene.childofwinter.chance = math.random()
		
		if scene.childofwinter.chance < 0.50 then -- No encounter this playthrough
			return
		end
		
		scene.childofwinter.visible = true
		scene.player.canbecontrolled = false
		
	-- Third last map of the game, where we can encounter the Child of Winter, after fade in	
	elseif game.getCurrentScene() == scene and scene.currentmapindex == (#scene.maps - 3) then
		
		if scene.childofwinter.chance < 0.50 then -- No encounter this playthrough
			scene.player.canbecontrolled = true
			return
		end
		
		-- Read these "acts" from bottom to top
		
		local finalAct = function()
			-- scene.player.canbecontrolled = true
			-- game.audio.loop('caves')
			if not game.snow.on then
				game.snow.clear()
				game.snow.fillscreen()
				game.snow.on = true
			end
			
			local fader = game.getScene('fader')
			fader.setTarget('ending')
			game.setCurrentScene('fader')
		end
		
		local act11 = function()
			game.timer.execute(function()
				scene.dialog.endevent = finalAct
				-- scene.player.direction = 'up'
				scene.dialog.display(game.text.childofwinter7,1)
			end,0.5)
		end
	
		-- Flash 4 - White is permanent
		local act10 = function()
			scene.flash.endevent = act11
			scene.flash.notify = function()
				--game.audio.play('flash')
				scene.flash.direction = 'permanent' -- stops white from fading out
				scene.childofwinter.visible = false
				scene.flash.endevent()
			end
			
			scene.flash.direction = 'up'
			scene.flash.alpha = 1
			scene.flash.on = true
		end
	
		-- You're silent
		local act9 = function()
			scene.dialog.endevent = act10
			scene.dialog.display(game.text.childofwinter6,1)
		end
	
		-- Flash 3 - girl moves forward
		local act8 = function()
			scene.flash.endevent = act9
			scene.flash.notify = function()
				--game.audio.play('flash')
				scene.childofwinter.yoffset = scene.tileset.tilewidth
			end
			
			scene.flash.direction = 'up'
			scene.flash.alpha = 1
			scene.flash.on = true
		end
	
		-- You're in pain
		local act7 = function()
			scene.dialog.endevent = act8
			scene.dialog.display(game.text.childofwinter5,1)
		end
	
		-- Flash 2 - you're on the ground now
		local act6 = function()
			scene.flash.endevent = act7
			scene.flash.notify = function()
				--game.audio.play('flash')
				scene.player.direction = 'onground'
			end
			
			scene.flash.direction = 'up'
			scene.flash.alpha = 1
			scene.flash.on = true
		end
	
		-- Your head hurts now
		local act5 = function()
			scene.dialog.endevent = act6
			scene.dialog.display(game.text.childofwinter4,1)
		end
	
		-- Flash 1
		local act4 = function()
			scene.flash.direction = 'up'
			scene.flash.alpha = 1
			--scene.flash.notify = function() game.audio.play('flash') end
			scene.flash.sfx = 'flash'
			scene.flash.endevent = act5
			scene.flash.on = true
		end
		
		local act3 = function()
			-- Try talking to the child of winter
			--game.timer.execute(function()
				scene.dialog.endevent = act4
				scene.dialog.display(game.text.childofwinter3,1)
			--end,0.25)
		end
		
		local act2 = function()
			-- Take 1 step forward
			game.timer.execute(function()
				scene.player.tiley = scene.player.tiley - 1
			end,0.25)
			
			-- Try talking to the child of winter
			game.timer.execute(function()
				scene.dialog.endevent = act3
				scene.dialog.display(game.text.childofwinter2,1)
			end,0.75)
		end
		
		local act1 = function()
			-- Take 2 steps forward
			game.timer.execute(function()
				scene.player.tiley = scene.player.tiley - 2
			end,0.25)
			
			-- Notice the child of winter
			game.timer.execute(function()
				scene.dialog.endevent = act2
				scene.dialog.display(game.text.childofwinter1,1)
			end,1.0)
		end
		
		act1(); game.audio.stopAll()
		
		
	-- After the fall
	elseif scene.currentmapindex == 2 then

		scene.player.direction = 'onground'
		
		scene.dialog.endevent = function ()
			scene.dialog.visible = false
			scene.player.direction = 'up' -- better to face up since onground sprite faces up
			scene.player.canbecontrolled = true
			
			game.audio.stopAll()
			game.audio.loop('caves')
		end
		
		scene.dialog.display(game.text.afterfall,2)

	-- Initial "I need to find a way out of here!" dialogue
	elseif game.getCurrentScene() == scene and scene.currentmapindex == 3 then
		
		scene.player.tiley = scene.player.tiley - 3
		
		game.timer.execute(function()
			scene.dialog.endevent = function ()
				scene.player.canbecontrolled = true
			end
			scene.dialog.display(game.text.puzzle1,2)
		end,0.65)
		
	-- Last map of game, before fade in
	elseif game.getCurrentScene() ~= scene and scene.currentmapindex == #scene.maps then
		
		-- Need to ensure snow is on before fade in animation begins
		game.audio.stopAll()
		if not game.snow.on then
			game.snow.clear()
			game.snow.fillscreen()
			game.snow.on = true
		end
		
	-- Last map of game, after fade in
	elseif game.getCurrentScene() == scene and scene.currentmapindex == #scene.maps then
		
		scene.player.canbecontrolled = false

		scene.player.tiley = scene.player.tiley + 1
		
		game.timer.execute(function()
			
			scene.dialog.endevent = function ()
				-- goto credits
				local fader = game.getScene('fader')
				fader.setTarget('ending')
				game.setCurrentScene('fader')
			end
			
			scene.dialog.display(game.text.ending,3)
		end,0.65)

	-- Player is at menu screen but scene-main is being rendered in the background
	elseif scene.currentmapindex == 1 then
	
		scene.player.direction = 'down'
		
		if not game.snow.on then
			game.snow.clear()
			game.snow.fillscreen()
			game.snow.on = true
		end

	-- All other maps
	elseif game.getCurrentScene() == scene then
		scene.player.canbecontrolled = true
	end

end


function manager.tiletrigger()
	-- NOT CURRENTLY USED ANYWHERE
end

return manager