
-- [turtlesort.com, 2012]

-- A dialog box for displaying text.
-- Usage:
--
-- dialog.display({
--     'line 1',
--     'line 2'
-- },2)

local dialog = {}

-- Dimensions and location on screen
dialog.width = 550
dialog.height = 80
dialog.x = love.graphics.getWidth()/2 - dialog.width/2
dialog.y = love.graphics.getHeight()/2 + dialog.height*2
dialog.padding = 10
dialog.visible = false

-- Text attributes
dialog.font = love.graphics.getFont()
dialog.text = {''} -- an array of strings
dialog.textindex = 1
dialog.visiblelines = 1

-- Only used for marquee-type displaying for text
dialog.charindex = {1}
dialog.chardelay = 0.01 -- 10ms per character
dialog.chartimer = 0


-- Displays text on the screen
function dialog.display(textArray, visiblelines)
	
	dialog.visible = false
	
	dialog.text = textArray
	dialog.textindex = 1
	
	dialog.charindex = {}
	dialog.chardelay = 0.01 -- 10ms per character
	dialog.chartimer = 0

	for i = 1, visiblelines do
		dialog.charindex[i] = 0
	end
	
	dialog.charindex[1] = 1
	
	if visiblelines then 
		dialog.visiblelines = visiblelines
	else
		visiblelines = 1
	end
	
	dialog.visible = true
end

function dialog.draw()

	if dialog.visible then
	
		love.graphics.setColor(20,20,20)
		love.graphics.rectangle('fill', dialog.x, dialog.y, dialog.width, dialog.height)
		
		if dialog.textindex <= #dialog.text then		
			love.graphics.setColor(255,255,255)
			love.graphics.setFont(dialog.font)
			
			for i = dialog.textindex, math.min(dialog.textindex + dialog.visiblelines - 1, #dialog.text) do
				if dialog.charindex[i - dialog.textindex + 1] > 0 then
					love.graphics.print(
						string.sub(dialog.text[i],1,dialog.charindex[i - dialog.textindex + 1]),
						dialog.x + dialog.padding, 
						dialog.y + dialog.padding + 25 * (i - dialog.textindex)
					)
				end
			end
		end
		
	end

end

function dialog.update(dt)

	if dialog.visible and dialog.textindex <= #dialog.text then

		dialog.chartimer = dialog.chartimer + dt
		
		if dialog.chartimer >= dialog.chardelay then -- Is it time to display the next character?
			dialog.chartimer = 0
			
			for i = 1, dialog.visiblelines do
			
				-- Check if the line exists first (we could be at the end of the text block)
				if not dialog.text[dialog.textindex + i - 1] then break end
			
				local linelength = string.len(dialog.text[dialog.textindex + i - 1])
			
				-- For each line, increase the charindex (essentially the number of characters to show) only when:
				--  1. there is at least 1 character already being shown
				--  2. there are still characters in the line that need to be shown
				if dialog.charindex[i]  > 0 and  not (dialog.charindex[i] == linelength) then
				   
				   -- Show next character.
					dialog.charindex[i] = dialog.charindex[i] + 1
					
					-- All characters for the current line are being shown. Display the first
					-- character of the next line (if there is one).
					if dialog.charindex[i] == linelength and dialog.visiblelines >= i + 1 then
						dialog.charindex[i + 1] = 1
					end
					
					break -- Do not process any other lines
				end
			end

		end
		
	end
end

function dialog.keypressed(key,unicode)

	if (key == ' ' or key == 'return') and dialog.visible and dialog.textindex <= #dialog.text then
		dialog.textindex = dialog.textindex + dialog.visiblelines
		
		for i = 1, dialog.visiblelines do
			dialog.charindex[i] = 0
		end
		
		dialog.charindex[1] = 1
		
		if dialog.textindex > #dialog.text then
			dialog.visible = false
			
			--
			if dialog.endevent then
				dialog.endevent()
			end
			--]]
			-- if there's nothing left close the dialog
		end
		
	end

end

return dialog