
-- [turtlesort.com, 2012]

-- A timer for executing tasks at a specified point
-- in the future. It doesn't do anything fancy like
-- recurring tasks.

local timer = {}
timer.tasks = {}


function timer.update(dt)

	local t = love.timer.getTime()
	for i = #timer.tasks, 1, -1 do
		
		local task = timer.tasks[i]
		
		if t - task.start >= task.time then
			task.execute()
			table.remove(timer.tasks, i)
		end
		
	end

end

-- 'func' is the anonymous function to execute, it
-- shouldn't have any paramters.
-- 'seconds' is the number of seconds until the function
-- should be executed.
function timer.execute(func,seconds)
	
	local task = {}
	
	task.execute = func
	task.time = seconds
	task.start = love.timer.getTime()
	
	table.insert(timer.tasks, task)

end


return timer