
-- [Ice Project 2012, turtlesort.com]

-- This is an intermediate screen for facilitating
-- fade-in/fade-out transitions between a pair of
-- screens. Intended to be used with a scenedirector.

local fader = {}
local fadeStart
local fadeAlpha
local state
local sceneDirector

-- Can be adjusted by caller
fader.fadeOutTime = 0.35 -- Amount of time in seconds spent fading out
fader.fadeInTime = 0.35  -- Amount of time in seconds spent fading in

function fader.setSceneDirector(director)
	sceneDirector = director
end

-- Make sure to set the target screen before
-- letting sceneDirector.display show this screen
-- nextID: id of the screen to fade-transition into
-- prevID: (optional) id of screen to fade-transition out of
function fader.setTarget(nextID, prevID)
	if not prevID then
		fader.previousScreen = sceneDirector.getCurrentScene()
	else
		fader.previousScreen = sceneDirector.getScene(prevID)
	end
	fader.nextScreen = sceneDirector.getScene(nextID)
	fader.nextID = nextID
end

-- Resets state to 'fadein'
function fader.load()
	state = 'fadein'
	fadeStart = love.timer.getTime()
	fadeAlpha = 0
end

-- Gradually updates the state and alpha of fade rectangle
function fader.update(dt)

	local diff = love.timer.getTime()-fadeStart
	
	if diff <= fader.fadeOutTime then
		fadeAlpha = 255 * (diff/fader.fadeOutTime)
		
		if fader.previousScreen.update then
			fader.previousScreen.update(dt)
		end
	
	elseif diff > fader.fadeOutTime and state == 'fadein' then 
		state = 'fadeout'
		
		if fader.nextScreen.load then
			fader.nextScreen.load()
		end
		
	elseif diff > fader.fadeOutTime and diff <= fader.fadeOutTime+fader.fadeInTime then
		fadeAlpha = 255 - (255 * (diff-fader.fadeOutTime)/(fader.fadeInTime))
		
		if fader.nextScreen.update then
			fader.nextScreen.update(dt)
		end
		
	else
		state = 'stop'
		
		sceneDirector.setCurrentScene(fader.nextID)
	end
	

end

-- Draws the fade rectangle
function fader.draw()

	if state == 'fadein' then
		fader.previousScreen.draw()
	elseif state == 'fadeout' then
		fader.nextScreen.draw()
	end

	love.graphics.setColor(0,0,0,fadeAlpha)
	love.graphics.rectangle('fill',0,0,love.graphics.getWidth(),love.graphics.getHeight())
end

return fader